options ls=85 nodate;
* Design parameters;
%let h=0.001;  * Delta for finite difference derivative approximation;
%let paran=7;  * Number of parameters in the model;
%let nf=2;     * Number of fixed effect parameters;
%let cost=2;   * Cost function (1, no cost function, 2, user-specified function);
* Algorithm parameters;
%let convc=1e-9;
%let maximit=1000;
%let const1=2;
%let const2=1;
%let cmerge=5;
* PK parameters;
data para;
    input CL V vCL vV covCLV m s;
    datalines;
    0.211 5.50 0.0365 0.0949 0.0443 0.0213 8060
    ;
* All candidate points;
data cand;
    input x @@;
    datalines;
    0.083 0.25 0.5 0.75 1 2 3 4 5 6 12 24 36 48 72 144
    ;
* Number of time points in the final design;
data ks;
    input r @@;
    datalines;
    3 4 5
    ;
* Initial design;
data sample;
    input x1 x2 x3 x4 x5  w @@;
    datalines;
    0.083 0.5 4 24 144 1.0
    ;
    run;
* Output data set NDESIGN contains the design points;
* Output data set WEIGHTP contains the weights;
data ndesign;
    set sample;
    drop w;
data weight;
    set sample;
    keep w;
proc print data=sample label noobs;
    title 'The Initial Design';
    label w='Weight';
    run;
/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^|
|Find the optimal design using forward and backward procedures       |
|D-optimal criteria is used.                                         |
|N: the number of parameters in the design                           |
|Navail: the number of the available points in the design region     |
|C: the merge criteria                                               |
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^|
|Macro INFOD calculates the information matrix              |
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
%macro infod(nin, sensiout,call);
 %if &call=1 %then %do;
 * Find matrix of all possible N choose K data sets from Xdomain;
 nt=nrow(para);
 nX=nrow(Xdomain);
 maxK=max(K);
 do i=1 to nrow(K);
    Ki=K[i];
    matrixTi=Ts(nX,Ki,Xdomain);
    if maxK-Ki>0 then do;
        Jnt=J(nrow(matrixTi),maxK-Ki,0);
        matrixTi=matrixTi||Jnt;
    end;
    if i>1 then matrixTi[,1]=nrow(newx)+matrixTi[,1];
    if i=1 then newx=matrixTi;
    else newx=newx//matrixTi;
    * Calculate u(x,para) for all possible combinations of designs in Xdomain;
    do z=1 to nrow(matrixTi);
        t=(matrixTi[z,2:Ki+1]);
        t=(t[loc(t>0)])`;
        * Jacobian;
        Jac=JacSet(para,t,&nf,&h);
        fparat=fpara(para,t);
        fd=diag(fixed);
        SIT=JAC*fd*e*fd*JAC`;
        TD=DIAG(fparat`*fparat+SIT);
        * S matrix for design;
        Si=SIT+e1*TD+s2*I(Ki);
        iSi=inv(Si);
        * Information matrix u(x,para);
        ui=Mold(nt,Jac,para_p1,para_m1,&h,&nf,E,Ki,t,iSi,ranth,para,fd);
        C=1;
        if &cost=2 then C=mcost(t);
        ui=ui/C;
        * Create allm containing u(x,para) for all possible combinations;
        if (i=1 & z=1) then allm = J(nrow(ui),1,z)||ui;
        else allm = allm//(J(nrow(ui),1,matrixTi[z,1])||ui);
    end; * do z=1 to nrow(matrixTi);
end; * do i=1 to nrow(K);
  * Information matrix for initial design;
  do i=1 to nrow(xadd);
    t=xadd[i,];
    Ki=ncol(t);
    Jac=JacSet(para,t,&nf,&h);
    fparat=fpara(para,t);
    fd=diag(fixed);
    SIT=JAC*fd*e*fd*JAC`;
    TD=DIAG(fparat`*fparat+SIT);
    * S matrix for design;
    Si=SIT+e1*TD+s2*I(Ki);
    iSi=inv(Si);
    ui=weight[i]*Mold(nt,Jac,para_p1,para_m1,&h,&nf,E,Ki,t,iSi,ranth,para,fd);
    C=1;
    if &cost=2 then C=mcost(t);
    ui=ui/C;
    if i=1 then M=ui;
    else M=M+ui;
  end; * do i for initial design;
  designm=J(&paran,1,1)||M;
  invm=inv(M);
  nobs=nrow(newx);
%end;
    &sensiout=j(nobs,1,0);
    do v= 1 to nobs;
    pos=loc(allm[,1]=v); 
    test=allm[t(pos),2:&paran+1]; 
    &sensiout[v,]=trace(test*invm);
    end; 
%mend infod;
%macro doptimal(n);
    %infod(nobs,result,2);

    ncx=ncol(newx);
    dist=1;
    wp=&const1/(kk*&const2+ninit); 
    * add a new point, get design xadd;
    i=result[<:>];
    xpoint=newx[i,2:ncx];
    merge=0;
    weight=weight*(1-wp); 
    * merge close points ;
    nobs1=nrow(xadd);     
    do j=1 to nobs1 until (merge=1);
    pad_x = (ncx-1)-ncol(xadd);
    if pad_x>0 then xaddj=xadd[j,]||J(1,pad_x,0);
    if sqrt(ssq(xadd[j,]-xpoint))<=&cmerge*dist then do;
    M=(M-weight[j]/(1-wp)*
    designm[t(loc(designm[,1]=j)),2:&paran+1])*(1-wp)
    +(weight[j]+wp)*allm[t(loc(allm[,1]=i)), 2:&paran+1];
    weight[j]=weight[j]+wp;
    xadd[j,]=xpoint;
    designm[t(loc(designm[,1]=j)),2:&paran+1]
    =allm[t(loc(allm[,1]=i)), 2:&paran+1];
    merge=1;
    end; * if;
    end; * do j;
    if (merge=0) then do;
    pnew=wp;
    weight=weight//pnew;
    xadd=xadd//xpoint; 
    addm=j(&paran,1,nobs1+1)||allm[t(loc(allm[,1]=i)),2:&paran+1]; 
    designm=designm//addm;
    M=M*(1-wp)+pnew*allm[t(loc(allm[,1]=i)),2:&paran+1]; 
    end;
    invm=inv(M); 
    result=j(nobs1,1,0);
    do v= 1 to nobs1;
    pos=loc(designm[,1]=v); 
    test=designm[t(pos),2:&paran+1]; 
    result[v,]=trace(test*invm);
    end; 
    * delete a bad point, get xadd;
    nobs1=nrow(xadd);
    i=result[>:<]; 
    xdelet=xadd[i,];
    if weight[i]< wp then do; 
    a=weight[i]/(1-weight[i]);
    m=(m-weight[i]*designm[t(loc(designm[,1]=i)),2:&paran+1])
    *(1+a); 
    weight=weight*(1+a);
    if i=1 then do;
    weight=weight[i+1:nobs1,];
    xadd=xadd[i+1:nobs1,];
    designm=designm[t(loc(designm[,1]>i)),]; 
    designm[t(loc(designm[,1]>=i)),1]=designm[t(loc(designm[,1]>=i)),1]-1;
    end;
    else do;
    if i=nobs1 then do;
    weight=weight[1:i-1,];
    xadd=xadd[1:i-1];
    designm=designm[t(loc(designm[,1]<i)),]; 
    end;
    else do;
    weight=weight[1:i-1,]//weight[i+1:nobs1,];
    xadd=xadd[1:i-1,]//xadd[i+1:nobs1,];
    designm=designm[t(loc(designm[,1]<i)),]//designm[t(loc(designm[,1]>i)),]; 
    designm[t(loc(designm[,1]>=i)),1]=designm[t(loc(designm[,1]>=i)),1]-1;
    end;
    end;

    end;  
    else do;
    m=m*(1+wp)-wp*designm[t(loc(designm[,1]=i)),2:&paran+1];
    weight=weight*(1+wp);
    weight[i]=weight[i]-wp;
    end;  

    invm=inv(m);
    criter=log(det(invm)); 
%mend doptimal;

/*^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^|
|# of parameters is defined in Macro DOLOOP     |
^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^*/
%macro doloop;
proc iml;

*for a PK model the function f(para) must be entered;
    start fpara(para,t);    * function to estimate the function for given para vector;
        k12=0.400;
        k21=0.345;
        CL=para[1]; V=para[2];
        k10=CL/V;
        l2=0.5*((k21+k12+k10)-sqrt((k12+k21+k10)**2-4*k21*k10));
        l1=k21*k10/l2;
        A=((k21-l1)*exp(-l1*t))/(l2-l1);
        B=((k21-l2)*exp(-l2*t))/(l1-l2);
        f=(0.60*70*1000/V)*(A+B);       * value of function to return;
        return(f);
    finish fpara;

* Ts creates a matrix of indeces for all N choose K combinations;
* This matrix is called CombAll;
* It then returns to the main program the matrix t which is all of the values of Xdomain;
* indexed by Comball;
start Ts(nX,Ki,Xdomain);
    rc=0;
    NchooseK=Fact(nX)/(Fact(Ki)*Fact(nX-Ki));
    CombAll=J(NchooseK,Ki,0);
    ij=1;
    Current=1:Ki;
    CombAll[1,]=Current;
    do while(rc=0);
        reset1=0;
        next=Current;
        do i=Ki to 2 by -1;
          if (Current[i]=(nX-Ki+i) & Current[i-1]<(nX-Ki+i-1)) then do;
            reset1=1;
            next[i-1]=next[i-1]+1;
            do ii=i to Ki;
                next[ii]=next[ii-1]+1;
            end; 
          end; 
        end; 
    if reset1=0 then do;
      do i=1 to Ki;
        if (i=1 & Current[i]=(nX-Ki+1)) then rc=1;
        if (i<Ki & Current[i]>i) then do;
            if (Current[i+1]=(nX-Ki+i+1)) then next[i]=Current[i]+1;
        end;
        if (i=Ki & Current[i]<nX) then next[i]=Current[i]+1;
      end; 
    end; 
    if rc=0 then do;
        ij=ij+1;
        CombAll[ij,]=next;
        current=next;
    end; 
end; 
t=(1:NchooseK)`||shape(Xdomain[CombAll],NchooseK,Ki);
return(t);
finish Ts;

start mcost(t);
        * t is vector sampling times;
        Cv = 1;
        Cs = 0.3;
        C = Cv + Cs*nrow(t[loc(t>0)]);
        return(C);
finish mcost;

start fplus(ii,para_p1,h);
        para_f1=para_p1;
        para_f1[,ii]=para_p1[,ii]+&h;
        return(para_f1);
finish fplus;

start fminus(ii,para_m1,h);
        para_f1=para_m1;
        para_f1[,ii]=para_m1[,ii]-&h;
        return(para_f1);
finish fminus;

start Esetup(ranth,nf);
    E=diag(ranth[1:&nf]);
    indexr=&nf+1;
    do ii=1 to (&nf-1);
      do jj=(ii+1) to &nf;
        rv=ranth[indexr];
        E[ii,jj]=rv;
        E[jj,ii]=rv;
        indexr=indexr+1;
      end;
    end;
    return(E);
finish Esetup;
        
start JacSet(para,X,nf,h);
     pK=ncol(X);
     do i=1 to pK;
        t=X[,i];

        do ii=1 to &nf;
           * call subroutine to find Jacobian;
           dydt=Jacob(ii,para,&h,t);       * dydt is scalar for d-eta/d-para, fixed part;
           if ii=1 then Jrow=dydt;
           else Jrow=Jrow||dydt;           * Jrow is 1xnf;

        end; 
     if pK=1 then Jac=Jrow;
     else Jac=Jac//Jrow;                   * Jac is Kxnf;
     end; 

  return(Jac);
finish JacSet;

start Mold(nt,Jac1,para_p1,para_m1,h,nf,E,zK,t,iSi,ranth,para,fd);
    Mi=J(nt,nt,0);
    varm=para[nt-1];        * multiplicative term;
    do ii=1 to nt;
      do jj=ii to nt;
      * calculate 2nd derivatives of Jacobian for estimate of variance for ii;
      if ii<=&nf then do;
        d_eta1=Jac1[,ii];
        d_eta_v=higherd(ii,para_p1,para_m1,&h,&nf,t); * d_eta_v is zx2nf;
        d_eta_l=d_eta_v[,1:&nf];     
        d_eta_r=d_eta_v[,(&nf+1):2*&nf];
        * d_eta_l, d_eta_r, are p*&nf;
        * d_eta is d/d-para(d-eta*E*d-eta);
        * for multiplicative term;
        fparad=Jacob2(ii,para,&h,t);
        fparap=fparad[,1];  
        fparam=fparad[,2];
        fd_p1=diag(para_p1[ii,]);
        fd_m1=diag(para_m1[ii,]);
        S1_L = d_eta_l*fd_p1*E*fd_p1*d_eta_l`;
        S1_R = d_eta_r*fd_m1*E*fd_m1*d_eta_r`;
        S1_Delta = S1_L - S1_R;
        VARMDS1=varm*DIAG(fparap*fparap`-fparam*fparam` + S1_Delta);
        S1=(1/(2*&h))*(S1_Delta + VARMDS1); *S1 is ;
      end;  * if ii<=&nf;
      else do;
        d_eta1=J(zK,1,0);
        if ii<(nt-1) then do;
            fx=fpara(para,t);
            dydSi=higherm(ii,&nf,ranth,&h,Jac1,fd);
            * S1=varm*DIAG(dydSi+fx`*fx);
            S1=dydSi+varm*DIAG(dydSi);
        end;    * if ii<(nt-1);
        if ii=(nt-1)then do;
            fx=fpara(para,t);
            S1=DIAG(fx`*fx + Jac1*fd*E*fd*Jac1`);
        end;    * if ii=(nt-1);
        if ii=nt then S1=I(zK);
      end;  * else do;

    * calculate 2nd derivatives of Jacobian for estimate of variance for jj;
    if jj<=&nf then do;
        d_eta2=Jac1[,jj];
        d_eta_v=higherd(jj,para_p1,para_m1,&h,&nf,t); * d_eta_v is zx2nf;
        d_eta_l=d_eta_v[,1:&nf];
        d_eta_r=d_eta_v[,(&nf+1):2*&nf];
        * d_eta_l, d_eta_r, are p*&nf;
        * d_eta is d/d-para(d-eta*E*d-eta);
        * for multiplicative term;
        fparad=Jacob2(jj,para,&h,t);
        fparap=fparad[,1];
        fparam=fparad[,2];
        fd_p1=diag(para_p1[jj,]);
        fd_m1=diag(para_m1[jj,]);
        S2_L = d_eta_l*fd_p1*E*fd_p1*d_eta_l`;
        S2_R = d_eta_r*fd_m1*E*fd_m1*d_eta_r`;
        S2_Delta = S2_L - S2_R;
        VARMDS2=varm*DIAG(fparap*fparap`-fparam*fparam` + S2_Delta);
        S2=(1/(2*&h))*(S2_Delta + VARMDS2); *S2 is ;
    end;    * if jj<= &nf;
    else do;
        d_eta2=J(zK,1,0);
        if jj<(nt-1) then do;
            fx=fpara(para,t);
            dydSi=higherm(jj,&nf,ranth,&h,Jac1,fd);
            * S2=varm*DIAG(dydSi+fx`*fx);
            S2=dydSi+varm*DIAG(dydSi);
        end;    * if jj<(nt-1);
        if jj=(nt-1) then do;
            fx=fpara(para,t);
            S2=DIAG(fx`*fx + Jac1*fd*E*fd*Jac1`);
        end;    * if jj=(nt-1);
        if jj=nt then S2=I(zK);
    end;    * else do;

    VhatL=d_eta1`*iSi*d_eta2;
    VhatR=0.5*trace(iSi*S1*iSi*S2);
    Vhat=VhatL+VhatR;
    Mi[ii,jj]=Vhat;
    Mi[jj,ii]=Vhat;
end; * jj;
end; *ii;
return(Mi);
finish Mold;

start Jacob(ii,para,h,t);               * derivative approximation for J;
        dydt=0;
        do jj=1 to -1 by -2;            * outer loop for first derivative;
            para_d=para;                * for paras adjusted for derivative approximation;
            para_d[ii]=para_d[ii]+jj*&h;
            fhat=fpara(para_d,t);       * call to fit specified function;
            der=fhat/(2*&h);
            dydt=dydt+(jj*der);
        end; * do jj;
        return(dydt);
finish Jacob;

start Jacob2(ii,para,h,t);              * derivative approximation for J for multiplicative term;
        do jj=1 to -1 by -2;            * outer loop for first derivative;
            para_d=para;                * for paras adjusted for derivative approximation;
            para_d[ii]=para_d[ii]+jj*&h;
            fhat=fpara(para_d,t);       * call to fit specified function;
            if jj=1 then dydtm=fhat`;
            else dydtm=dydtm||fhat`;        * dydtm = +&h in first column, -&h is second column;
        end; * do jj;
        return(dydtm);
finish Jacob2;

start higherd(ii,para_p1,para_m1,h,nf,t);       * calculates 2nd derivatives of Jacobian for estimate of fixed part of variance;
    fpL=fplus(ii,para_p1,&h);                   * "plus" part for random part derivatives;
    fmL=fplus(ii,para_m1,&h);                   * "minus" part for random part derivatives;
    fpR=fminus(ii,para_p1,&h);                  * "plus" part for random part derivatives;
    fmR=fminus(ii,para_m1,&h);                  * "minus" part for random part derivatives;

    do kk=1 to &nf;
        * Left hand side of overall derivative ("big plus" part);
        A1=(fpara(fpL[kk,],t))`;
        A2=(fpara(fmL[kk,],t))`;
        if kk=1 then d_eta_l=(A1-A2)/(2*&h);
        else d_eta_l=d_eta_l||((A1-A2)/(2*&h));

        * Right hand side of overall derivative ("big minus" part);
        A1=(fpara(fpR[kk,],t))`;
        A2=(fpara(fmR[kk,],t))`;

        if kk=1 then d_eta_r=(A1-A2)/(2*&h);
        else d_eta_r=d_eta_r||((A1-A2)/(2*&h));
    end; *do kk;
        d_eta=d_eta_l||d_eta_r;
        return(d_eta);
finish higherd;

start higherm(ii,nf,ranth,h,Jrow,fd);
            ranth_m1=ranth;
            ranth_p1=ranth;
            ranth_m1[ii-&nf]=ranth[ii-&nf]-&h;
            ranth_p1[ii-&nf]=ranth[ii-&nf]+&h;
            Sm1=Esetup(ranth_m1,&nf);
            Sp1=Esetup(ranth_p1,&nf);
            dydSa=(Jrow*fd*Sp1*fd*Jrow`-Jrow*fd*Sm1*fd*Jrow`)/(2*&h);
         return(dydSa);
finish higherm;

* MAIN PART OF IML PROCESS;

 use ndesign;
        read all into xadd;
        close ndesign;  * initial design;
        use weight;
        read all var{w} into weight;
        close weight;   * weights for initial design;
        use para;
        read all into para;
        use Ks;
        read all into K;
        use cand;
        read all into Xdomain;

    nX=nrow(Xdomain);
    maxT=max(Xdomain);

if (max(K)-ncol(xadd) > 0) then xadd=xadd||J(nrow(xadd),max(K)-ncol(xadd),0);

para=t(para);
ranth=para[&nf+1:&paran-2];             * vector of random para's;
E=Esetup(ranth,&nf);                    * covariance matrix of para's;
e1=para[&paran-1];                      * multiplicative term;
s2=para[&paran];                        * additive term;
fixed=para[1:&nf];
para_p1=repeat(fixed,1,&nf)`+&h*I(&nf);     * for para_i + &h;
para_m1=repeat(fixed,1,&nf)`-&h*I(&nf);     * for para_i - &h;

   M=j(&paran,&paran,0);
  ninit=nrow(xadd);
  %infod(ninit,sensi1,1); 
   D=invm;
   ided=det(D);
   print 'Matrix D for the initial design'; print  D[format=7.3];
   print 'Determinant of the covariance matrix D (initial design)'; print ided;

    create sensione from sensi1[colname={sensi}];
    append from sensi1;
    close sensione;

    num=&maximit;
  do kk=1 to num until(stop=1);
    stop=0;
    s=kk;    
   %doptimal(&paran); 
    if kk=1 then critout=criter;
    if kk>1 then do;
     critout=critout//criter;
       out=abs((critout[kk]-critout[kk-1])/critout[kk-1]); * print out;
         if out[1]<&convc then do;  s=kk;          
          de=exp(criter); stop=1;
         end;
     end;
   end;

   D=invm;
   print 'Matrix D for the optimal design'; print D[format=7.3];
   detd=det(D);
          print 'Determinant of the covariance matrix D (optimal design)'; print detd;
  %infod(nobs,sensi2,2);

    create sensif from sensi2[colname={sensi}];
    append from sensi2;
    close sensif;
    create weight from weight[colname={w}];
    append from weight;
    close weight;
    create ndesign from xadd;
    append from xadd;
    close ndesign;
    icn=nrow(critout);
    newcol=t((1:icn));
    critout=newcol||critout;
    create crit from critout[colname={iter crit}];
    append from critout;
    close crit;
quit;
%mend doloop;
%doloop;
proc print data=ndesign noobs;
    title 'Optimal design';
proc print data=weight noobs;
    title 'Optimal weights';
    run;

