options ls=85 nodate;
%include 'D:\PharmStatistics\Draft\Chapter06\OptimalDesign2.sas';
* Design parameters;
%let points=do(0,8,8/5);
%let weights=repeat(1/6,1,6);
%let grid=do(0,8,8/400);
%let parameter={4 -0.49 3.9 0.15};
* Number of parameters;
%let paran=4;
* Algorithm parameters;
%let convc=1e-9;   
%let maximit=1000; 
%let const1=2;     
%let const2=1;     
%let cmerge=5;     
* Calculation of individual components of the information matrix;
%macro devpsi(matrix,derivative);
    nm=nrow(&matrix);
    out=j(nm,1,0);
    do r=1 to nm;
    a=&matrix[r,1];
    start f_model(a) global(r,&matrix);
    f=digamma(a);
    return(f);
    finish f_model;
    call nlpfdd(crit,grad,hess,"f_model",a);
    out[r,]=grad;
    end; 
    &derivative=out;
%mend devpsi;
%macro infoele(matrix,eleo11,eleo12,eleo13,eleo14,eleo22,eleo23,eleo24,
    eleo33,eleo34,eleo44); 
    nm=nrow(&matrix);
    p=exp(parameter[1]*j(nm,1,1)+parameter[2]*&matrix);
    q=exp(parameter[3]*j(nm,1,1)+parameter[4]*&matrix);
    pq=p+q;
    %devpsi(pq,ddipq);
    %devpsi(p,ddip);
    %devpsi(q,ddiq);
    &eleo11=p##2#(ddip-ddipq);
    &eleo12=&eleo11#&matrix;
    &eleo13=-ddipq#p#q;
    &eleo14=&eleo13#&matrix;
    &eleo22=&eleo12#&matrix;
    &eleo23=&eleo13#&matrix;
    &eleo24=&eleo14#&matrix;
    &eleo33=q##2#(ddiq-ddipq);
    &eleo34=&eleo33#&matrix;
    &eleo44=&eleo34#&matrix;
%mend infoele;
%macro indinfo(name,totaln,allm);
    allm=j(1,&paran+1,0);
    do index=1 to &totaln;
    indm=j(&paran,&paran,0);
    %do h=1 %to &paran;
    %do e=&h %to &paran;
    indm[&h,&e]=&name&h&e[index,];
    indm[&e,&h]=&name&h&e[index,];
    %end;
    %end;
    indm=j(&paran,1,index)||indm;
    allm=allm//indm;
    end;
    &allm=allm[2:nrow(allm),];
%mend indinfo; 
%macro info(weight,name,mout);
    &mout=j(&paran,&paran,0);
    %do h=1 %to &paran;
    %do e=&h %to &paran;
    &mout[&h,&e]=(&weight#&name&h&e)[+,];
    &mout[&e,&h]=&mout[&h,&e];
    %end;
    %end;
%mend info;
* Calculation of the information matrix;
%macro infod(nin,sensit,call);
    %if &call=1 %then %do;
        ninit=nrow(design);
        %infoele(design,sele11,sele12,sele13,sele14,sele22,sele23,sele24,
        sele33,sele34,sele44); 
        %indinfo(sele,ninit,designm); 
        %info(weight,sele,m); 
        %infoele(grid,ele11,ele12,ele13,ele14,ele22,ele23,ele24,
        ele33,ele34,ele44); 
        %indinfo(ele,nobs,allm); 
        invm=inv(m);
    %end;
    &sensit=j(nobs,1,0);
    do v=1 to nobs;
        pos=loc(allm[,1]=v); 
        test=allm[t(pos),2:&paran+1]; 
        &sensit[v,]=trace(test*invm);
    end; 
%mend infod;
%OptimalDesign2;
data initial;
    set initial;
    label p='Weight' x='X';
data optimal;
    merge optimal;
    label p='Weight' x='X';
proc sort data=optimal;
    by x;
proc print data=initial noobs label; 
    title "Initial design";
    format p x 6.3;
    var p x;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x 6.3;
    var p x;
proc print data=ddet noobs label; 
    title "Determinants of the variance-covariance matrices";
    format initial optimal 6.4;
proc print data=dinitial noobs label; 
    title "Variance-covariance matrix, initial design";
    format col1 col2 col3 col4 6.3;
proc print data=doptimal noobs label; 
    title "Variance-covariance matrix, optimal design";
    format col1 col2 col3 col4 6.3;
data initial;
    set initial;
    stage=1;    
data optimal;
    set optimal;
    stage=2;    
data design;
    set initial optimal;
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig05a.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Weight")
    order=(0 to 0.6 by 0.2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(0 to 8 by 2)
    width=1;
symbol1 i=needle value=circle color=black height=5;
symbol2 i=needle value=dot color=black height=5;
proc gplot data=design;
    plot p*x=stage/vaxis=axis1 haxis=axis2 frame nolegend;
    run;
    quit;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig05b.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Sensitivity")
    order=(2 to 8 by 2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(0 to 8 by 2)
    width=1;
symbol1 i=join value=none color=black line=20 width=3;
symbol2 i=join value=none color=black line=1 width=3;
proc gplot data=sensit;
    plot (initial optimal)*point/ overlay vaxis=axis1 haxis=axis2
    frame nolegend vref=4 lvref=34;
    run;
    quit;
