options ls=85 nodate;
%include 'D:\PharmStatistics\Draft\Chapter06\OptimalDesign3.sas';
*%include 'U:\Work\SASbook\Alexs_program\OptimalDesign3.sas';

%macro DesignParameters;
    * Initial design: Design points;
    * Four equally spaced doses for each drug;
    dosen=4;
    * Total number of points;
    dosem=dosen*(dosen+1)/2;
    points=j(dosem,2,0);
    k=0;
    do i=1 to dosen;         
        do j=1 to dosen-i+1;
            k=k+1;
            points[k,1]=0.1+(i-1)*9.9/(dosen-1);
            points[k,2]=0.1+(j-1)*9.9/(dosen-1);
        end;
    end;  
    * Initial design: Weights;
    * Equally weighted points;
    weights=t(repeat(1/10,1,10));
    * Grid;
    * 26 equally spaced points on each axis;
    gridn=26;
    gridm=gridn*(gridn+1)/2;
    grid=j(gridm,2,0);
    k=0;
    do i=1 to gridn;         
        do j=1 to gridn-i+1;
            k=k+1;
            grid[k,1]=0.1+(i-1)*9.9/(gridn-1);
            grid[k,2]=0.1+(j-1)*9.9/(gridn-1);
        end;
    end;        
    * Parameter values;
    para={1 1 1 1 1 1 1 0.6 0.4};
%mend DesignParameters;
* Number of parameters;
%let paran=9; 
* Algorithm parameters;
%let convc=1e-9;   
%let maximit=500;  
%let const1=1;     
%let const2=1;     
%let cmerge=3;
%macro infoele(matrix,name,ele77,ele78,ele79,ele88,ele89,ele99); 
    nm=nrow(&matrix);
    one=j(nm,1,1);
    zero=j(nm,3,0);
    sigma2=exp(para[7]+para[8]#&matrix[,1]+para[9]#&matrix[,2]);
    meanf=one||&matrix||&matrix[,1]#&matrix[,2]||&matrix[,1]##2||&matrix[,2]##2||zero;
    varf=one||&matrix;
    %let st=1;
    %do q=1 %to 9;
    &name&st&q=meanf[,&q]/sigma2;
    %end;
    %do o=2 %to 6;
    %do q=&o %to 9;
    &name&o&q=&name&st&q#meanf[,&o];
    %end;
    %end;
    &ele77=one*0.5;
    &ele78=0.5*&matrix[,1];
    &ele79=0.5*&matrix[,2];
    &ele88=&ele78#&matrix[,1];
    &ele89=&ele79#&matrix[,1];
    &ele99=&ele79#&matrix[,2];
%mend infoele;
%macro info(weight,name,mout);
    &mout=j(&paran,&paran,0);
    %do h=1 %to &paran;
    %do e=&h %to &paran;
    &mout[&h,&e]=(&weight#&name&h&e)[+,];
    &mout[&e,&h]=&mout[&h,&e];
    %end;
    %end;
%mend info;
%macro indinfo(name,totaln,allm);
    allm=j(1,&paran+1,0);
    do index=1 to &totaln;
    indm=j(&paran,&paran,0);
    %do h=1 %to &paran;
    %do e=&h %to &paran;
    indm[&h,&e]=&name&h&e[index,];
    indm[&e,&h]=&name&h&e[index,];
    %end;
    %end;
    indm=j(&paran,1,index)||indm;
    allm=allm//indm;
    end;
    &allm=allm[2:nrow(allm),];
%mend indinfo; 
%OptimalDesign3;
data initial;
    set initial;
    label p='Weight' x='Drug A' z='Drug B';
data optimal;
    merge optimal;
    label p='Weight' x='Drug A' z='Drug B';
proc sort data=optimal;
    by x;
proc print data=initial noobs label; 
    title "Initial design";
    format p x z 4.2;
    var p x z;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x z 4.2;
    var p x z;
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig07a.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Dose (Drug A)")
    order=(0 to 10 by 2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose (Drug B)")
    order=(0 to 10 by 2)
    width=1;
symbol1 i=none value=dot color=black height=1;
proc gplot data=grid;
    plot z*x/vaxis=axis1 haxis=axis2 frame;
    run;
    quit;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig07b.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Dose (Drug A)")
    order=(0 to 10 by 2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose (Drug B)")
    order=(0 to 10 by 2)
    width=1;
symbol1 i=none value=circle color=black height=5;
proc gplot data=initial;
    plot z*x/vaxis=axis1 haxis=axis2 frame;
    run;
    quit;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig07c.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Dose (Drug A)")
    order=(0 to 10 by 2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose (Drug B)")
    order=(0 to 10 by 2)
    width=1;
symbol1 i=none value=dot color=black height=5;
proc gplot data=optimal;
    plot z*x/vaxis=axis1 haxis=axis2 frame;
    run;
    quit;
