options ls=85 nodate;
%include 'D:\PharmStatistics\Draft\Chapter06\OptimalDesign4.sas';
* Design parameters;
%let points={0 0.333 0.667 1};
%let weights={0.25 0.25 0.25 0.25};
%let grid=do(0,1,1/400);
%let parameter={-0.9 10 -1.2 1.6 0.5};
* Number of parameters;
%let paran=5; 
* Algorithm parameters;
%let convc=1e-9;   
%let maximit=1000;  
%let const1=1;     
%let const2=1;     
%let cmerge=5;
%macro infoele(matrix,allmout); 
    nm=nrow(&matrix);
    mu1=parameter[1]+parameter[2]*&matrix[,1];
    mu2=parameter[3]+parameter[4]*&matrix[,1];
    u1=(mu2-parameter[5]*mu1)/sqrt(1-parameter[5]**2);
    u2=(mu1-parameter[5]*mu2)/sqrt(1-parameter[5]**2);
    p11=probbnrm(mu1,mu2,parameter[5]);
    bigphi1=probnorm(u1);
    bigphi2=probnorm(u2);
    sphi1=pdf('normal',mu1);
    sphi2=pdf('normal',mu2);
    f=j(nm,1,1)||&matrix[,1];
    biprob=exp((mu1##2+mu2##2-2*parameter[5]#mu1#mu2)/(-2*(1-parameter[5]**2)))
    /2/3.1415/sqrt(1-parameter[5]**2);
    p1d=probnorm(mu1); 
    pd1=probnorm(mu2); 
    p10=p1d-p11;
    p01=pd1-p11;
    p00=1-pd1-p1d+p11; 
    allm=j(1,&paran+1,0);
    do ind=1 to nm;
        indm=j(&paran,&paran,0); 
        lp=p11[ind]//p10[ind]//p01[ind]; 
        P=diag(lp); 
        pp=P-lp*t(lp); if det(P)<0.000001 then pp=pp+I(3)*0.000001; 
        mat1=diag(sphi1[ind]||sphi2[ind]);
        mat2=((bigphi1[ind])||1-bigphi1[ind]||-bigphi1[ind])//
        (bigphi2[ind]||(-bigphi2[ind])||(1-bigphi2[ind]));
        left=mat1*mat2@f[ind,]`//(biprob[ind]||-biprob[ind]||-biprob[ind]);
        indm=left*inv(pp)*t(left);
        indm=j(&paran,1,ind)||indm;
        allm=allm//indm;
    end;
    &allmout=allm[2:nrow(allm),];
%mend infoele;
%macro info(weight,allmin,mout);
    &mout=j(&paran,&paran,0);
    numinf=nrow(&allmin)/&paran;
    do h=1 to numinf;
        &mout=&mout+weight[h,]*&allmin[(h-1)*&paran+1:h*&paran,2:&paran+1];
    end;
%mend info;
%macro infod(sensiout,call);
    ninit=nrow(design);
    %if &call=1 %then %do;
        %infoele(design,designm); 
        %info(weight,designm,m);  
        %infoele(grid,allm);
        invm=inv(m); 
    %end;
    &sensiout=j(nobs,1,0);
    do v=1 to nobs;
        pos=loc(allm[,1]=v); 
        test=allm[t(pos),2:&paran+1];
        &sensiout[v,]=trace(test*invm);
    end; 
%mend infod;
%OptimalDesign4;
data initial;
    set initial;
    label p='Weight' x='X';
data optimal;
    merge optimal;
    label p='Weight' x='X';
proc sort data=optimal;
    by x;
proc print data=initial noobs label; 
    title "Initial design";
    format p x 6.3;
    var p x;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x 6.3;
    var p x;
proc print data=ddet noobs label; 
    title "Determinants of the variance-covariance matrices";
    format initial optimal 8.0;
proc print data=dinitial noobs label; 
    title "Variance-covariance matrix, initial design";
    format col1 col2 col3 col4 col5 6.3;
proc print data=doptimal noobs label; 
    title "Variance-covariance matrix, optimal design";
    format col1 col2 col3 col4 col5 6.3;
data initial;
    set initial;
    stage=1;    
data optimal;
    set optimal;
    stage=2;    
data design;
    set initial optimal;    
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig08a.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Weight")
    order=(0 to 0.6 by 0.2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(0 to 1 by 0.2)
    width=1;
symbol1 i=needle value=circle color=black height=5;
symbol2 i=needle value=dot color=black height=5;
proc gplot data=design;
    plot p*x=stage/vaxis=axis1 haxis=axis2 frame nolegend;
    run;
    quit;
filename output "D:\PharmStatistics\Draft\Chapter06\ch06fig08b.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Sensitivity")
    order=(0 to 16 by 4)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(0 to 1 by 0.2)
    width=1;
symbol1 i=join value=none color=black line=20 width=3;
symbol2 i=join value=none color=black line=1 width=3;
proc gplot data=sensit;
    plot (initial optimal)*point/ overlay vaxis=axis1 haxis=axis2
    frame nolegend vref=5 lvref=34;
    run;
    quit;
