options ls=85 nodate;
%include 'D:\PharmStatistics\Chapter07\OptimalDesign1.sas';
* Design parameters;
%let points={-1 -0.333 0.333 1};
%let weights={0.25 0.25 0.25 0.25};
%let grid=do(-1,1,0.01);
%let parameter={1 3};
* Algorithm parameters;
%let convc=1e-7;   
%let maximit=200;  
%let const1=1;     
%let const2=1;     
%let cmerge=3;     
* G function;
%macro deriv(matrix,derivative);
    nm=nrow(&matrix);
    one=j(nm,1,1);
    fm=one||&matrix;
    gc=exp(0.5*fm*t(parameter))/(1+exp(fm*t(parameter)));
    &derivative=gc#fm;
%mend deriv;
* Information matrix;
%macro infod(nin,derin,sensit,call);
    %if &call=1 %then %do;
        %deriv(grid,newxder);
        %deriv(design,nder);
    %end;
    do i=1 to &nin;
        m=m+weight[i,]*(t(nder[i,])*nder[i,]);
    end;
    invm=inv(m);
    &sensit=vecdiag(&derin*invm*t(&derin));
%mend infod;
%OptimalDesign1;
data initial;
    set initial;
    label p='Weight' x='X';
data optimal;
    merge optimal;
    label p='Weight' x='X';
proc print data=initial noobs label; 
    title "Initial design";
    format p x 6.3;
    var p x;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x 6.3;
    var p x;
proc print data=ddet noobs label; 
    title "Determinants of the variance-covariance matrices";
    format initial optimal 5.1;
proc print data=dinitial noobs label; 
    title "Variance-covariance matrix, initial design";
    format col1 col2 5.1;
proc print data=doptimal noobs label; 
    title "Variance-covariance matrix, optimal design";
    format col1 col2 5.1;
data initial;
    set initial;
    stage=1;    
data optimal;
    set optimal;
    stage=2;    
data design;
    set initial optimal;
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Chapter07\ch07fig01a.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Weight")
    order=(0 to 0.6 by 0.2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(-1 to 1 by 0.5)
    width=1;
symbol1 i=needle value=circle color=black height=5;
symbol2 i=needle value=dot color=black height=5;
proc gplot data=design;
    plot p*x=stage/vaxis=axis1 haxis=axis2 frame nolegend;
    run;
    quit;
filename output "D:\PharmStatistics\Chapter07\ch07fig01b.eps";
goptions reset=all device=pslepsf rotate=portrait gaccess=gsasfile
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Sensitivity")
    order=(0 to 3 by 1)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Dose")
    order=(-1 to 1 by 0.5)
    width=1;
symbol1 i=join value=none color=black line=20 width=3;
symbol2 i=join value=none color=black line=1 width=3;
proc gplot data=sensit;
    plot (initial optimal)*point/ overlay vaxis=axis1 haxis=axis2
    frame nolegend vref=2 lvref=34;
    run;
    quit;
