options ls=85 nodate;
%include 'D:\PharmStatistics\Chapter07\OptimalDesign1.sas';
* Design parameters;
%let points=do(-0.024,6.215,6.239/9);
%let weights=repeat(0.1,1,10);
%let grid=do(-0.024,6.215,6.239/800);
%let parameter={15.03 1.31 530 1587};
* Algorithm parameters;
%let convc=1e-9;
%let maximit=1000;
%let const1=1;   
%let const2=1;     
%let cmerge=12;   
* G function;
%macro deriv(matrix,derivative);
    nm=nrow(&matrix);    
    one=j(nm,1,1);
    der1=(parameter[4]-parameter[3])/(1+(exp(&matrix[,1])/parameter[1])
        ##parameter[2])##2#(parameter[2]/parameter[1])#(exp(&matrix[,1])
        /parameter[1])##parameter[2];
    der2=-(parameter[4]-parameter[3])/(1+(exp(&matrix[,1])/parameter[1])
        ##parameter[2])##2#(exp(&matrix[,1])/parameter[1])##parameter[2]
        #log(exp(&matrix[,1])/parameter[1]);
    der3=1-1/(1+(exp(&matrix[,1])/parameter[1])##parameter[2]);
    der4=1/(1+(exp(&matrix[,1])/parameter[1])##parameter[2]);
    &derivative=der1||der2||der3||der4;
%mend deriv;
* Information matrix;
%macro infod(nin,derin,sensit,call);
    %if &call=1 %then %do;
        %deriv(grid,newxder);
        %deriv(design,nder);
    %end;
    do i=1 to &nin;
        m=m+weight[i,]*(t(nder[i,])*nder[i,]);
    end;
    invm=inv(m);
    &sensit=vecdiag(&derin*invm*t(&derin));
%mend infod;
%OptimalDesign1;
data initial;
    set initial;
    label p='Weight' x='X';
data optimal;
    merge optimal;
    label p='Weight' x='X';
proc print data=initial noobs label; 
    title "Initial design";
    format p x 6.3;
    var p x;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x 6.3;
    var p x;
proc print data=ddet noobs label; 
    title "Determinants of the variance-covariance matrices";
    format initial optimal 8.6;
proc print data=dinitial noobs label; 
    title "Variance-covariance matrix, initial design";
    format col1 col2 col3 col4 6.3;
proc print data=doptimal noobs label; 
    title "Variance-covariance matrix, optimal design";
    format col1 col2 col3 col4 6.3;
data initial;
    set initial;
    stage=1;    
data optimal;
    set optimal;
    stage=2;    
data design;
    set initial optimal;
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Chapter07\ch07fig03a.eps";
goptions reset=all device=pslepsf rotate=portrait 
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Weight")
    order=(0 to 0.6 by 0.2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Concentration (log scale)")
    order=(-0.03 to 6.5 by 1.05)
    width=1;
symbol1 i=needle value=circle color=black height=5;
symbol2 i=needle value=dot color=black height=5;
proc gplot data=design;
    plot p*x=stage/vaxis=axis1 haxis=axis2 frame nolegend;
    run;
    quit;
filename output "D:\PharmStatistics\Chapter07\ch07fig03b.eps";
goptions reset=all device=pslepsf rotate=portrait 
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Sensitivity")
    order=(2 to 7 by 1)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Concentration (log scale)")
    order=(-0.03 to 6.5 by 1.05)
    width=1;
symbol1 i=join value=none color=black line=20 width=3;
symbol2 i=join value=none color=black line=1 width=3;
proc gplot data=sensit;
    plot (initial optimal)*point/ overlay vaxis=axis1 haxis=axis2
    frame nolegend vref=4 lvref=34;
    run;
    quit;
