options ls=85 nodate;
%include 'D:\PharmStatistics\Chapter07\OptimalDesign1.sas';
* Design parameters;
%let points=do(-0.024,6.215,6.239/7);
%let weights=repeat(0.125,1,8);
%let grid=do(-0.024,6.215,6.239/400);
%let parameter={15.03 1.31 530 1587 0.5 1};
* Number of parameters;
%let paran=6;
* Algorithm parameters;
%let convc=1e-9;
%let maximit=1000;
%let const1=1;   
%let const2=1.2;     
%let cmerge=7.5;
* GR function;
%macro deriv(matrix,derivative);
    nm=nrow(&matrix);
    out=j(nm,&paran,0);
    do r=1 to nm;
        start f_model(parameter) global(r,&matrix);
            f=parameter[3]+(parameter[4]-parameter[3])/
            (1+(exp(&matrix[r,1])/parameter[1])**parameter[2]);
            return(f);
        finish f_model;
        call nlpfdd(crit,grad,hess,"f_model",parameter);
        out[r,]=grad;
    end;
    &derivative=out;
%mend deriv;
* GV function;
%macro derivs(matrix,derivative);
    nm=nrow(&matrix);
    out=j(nm,&paran,0);
    do r=1 to nm;
        start fs_model(parameter) global(r,&matrix);
            fs=parameter[5]*(parameter[3]+(parameter[4]-parameter[3])/
            (1+(exp(&matrix[r,1])/parameter[1])**parameter[2]))**parameter[6];
            return(fs);
        finish fs_model;
        call nlpfdd(crit,grad,hess,"fs_model",parameter);
        out[r,]=grad;
    end;
    &derivative=out;
%mend derivs;
* Information matrix;
%macro infod(nin,derin,sensit,call);
    %if &call=1 %then %do;
        %deriv(grid,newxder);
        %derivs(grid,newxsder);
        %deriv(design,nder);
        %derivs(design,nsder);
        newxder=newxder||newxsder;
        nder=nder||nsder;
    %end;
    dermean=nder[,1:&paran];
    ders=nder[,&paran+1:2*&paran];
    do i=1 to &nin;
        m=m+weight[i,]*(t(dermean[i,])*dermean[i,]+t(ders[i,])*ders[i,]);
    end;
    invm=inv(m);
    dermean=&derin[,1:&paran];
    ders=&derin[,&paran+1:2*&paran];
    &sensit=vecdiag(dermean*invm*t(dermean))+vecdiag(ders*invm*t(ders));
%mend infod;
%OptimalDesign1;
data initial;
    set initial;
    label p='Weight' x='X';
data optimal;
    merge optimal;
    label p='Weight' x='X';
proc sort data=optimal;
    by x;
proc print data=initial noobs label; 
    title "Initial design";
    format p x 6.3;
    var p x;
proc print data=optimal noobs label; 
    title "Optimal design";
    format p x 6.3;
    var p x;
proc print data=ddet noobs label; 
    title "Determinants of the variance-covariance matrices";
    format initial optimal 8.6;
proc print data=dinitial noobs label; 
    title "Variance-covariance matrix, initial design";
    format col1 col2 col3 col4 col5 col6 6.3;
proc print data=doptimal noobs label; 
    title "Variance-covariance matrix, optimal design";
    format col1 col2 col3 col4 col5 col6 6.3;
data initial;
    set initial;
    stage=1;    
data optimal;
    set optimal;
    stage=2;    
data design;
    set initial optimal;
    run;
%let vsize=2.5;
%let hsize=2.8;
%let font=hwpsl009;
%let fontsize=8;
filename output "D:\PharmStatistics\Chapter07\ch07fig04a.eps";
goptions reset=all device=pslepsf rotate=portrait 
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Weight")
    order=(0 to 0.6 by 0.2)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Concentration (log scale)")
    order=(-0.03 to 6.5 by 1.05)
    width=1;
symbol1 i=needle value=circle color=black height=5;
symbol2 i=needle value=dot color=black height=5;
proc gplot data=design;
    plot p*x=stage/vaxis=axis1 haxis=axis2 frame nolegend;
    run;
    quit;
filename output "D:\PharmStatistics\Chapter07\ch06fig04b.eps";
goptions reset=all device=pslepsf rotate=portrait 
    gsfname=output handshake=none gsfmode=replace gsflen=132 gprotocol=sasgpasc
    gunit=pt ftext=&font vsize=&vsize in hsize=&hsize in htext=&fontsize noborder;
* Vertical axis;
axis1
    minor=none
    value=(font=&font color=black height=&fontsize pt)
    label=(font=&font color=black angle=90 height=&fontsize pt "Sensitivity")
    order=(4 to 9 by 1)    
    width=1;
* Horizontal axis;
axis2   
    minor=none
    value=(font=&font color=black height=&fontsize pt)    
    label=(font=&font color=black height=&fontsize pt "Concentration  (log scale)")
    order=(-0.03 to 6.5 by 1.05)
    width=1;
symbol1 i=join value=none color=black line=20 width=3;
symbol2 i=join value=none color=black line=1 width=3;
proc gplot data=sensit;
    plot (initial optimal)*point/ overlay vaxis=axis1 haxis=axis2
    frame nolegend vref=6 lvref=34;
    run;
    quit;
